//
// Created by hoelshare on 21.05.17.
//

#ifndef GPN_LASERTAG_PLAYERMENU_HPP
#define GPN_LASERTAG_PLAYERMENU_HPP

#define min(a, b) ((a)<(b)?(a):(b))
#define max(a, b) ((a)>(b)?(a):(b))

#include "BadgeUI.h"

class PlayerListMenu : public Menu {
public:
    PlayerListMenu(int itemsPerPage = 3) : Menu(itemsPerPage) {
#ifdef DEBUG
        Serial.println("PlayerListMenu - init");
#endif
    };

    void setLeftAction(std::function<void(void)> leftAction) {
#ifdef DEBUG
        Serial.printf("Left Function set");
#endif
        this->leftAction = leftAction;
#ifdef DEBUG
        Serial.printf("Left Function setted");
#endif
    }

    void setRightAction(std::function<void(void)> rightAction) {
#ifdef DEBUG
        Serial.printf("Right Function set");
#endif
        this->rightAction = rightAction;
    }

    void setUpAction(std::function<void(void)> upAction) { this->upAction = upAction; }

    void setDownAction(std::function<void(void)> downAction) { this->downAction = downAction; }

    void setEnterAction(std::function<void(void)> enterAction) { this->enterAction = enterAction; }

    unsigned short getIndex() { return index; }

    void dispatchInput(JoystickState state) {
#ifdef DEBUG
        Serial.println("PlayerListMenu - dispatch");
#endif
        switch (state) {
            case JoystickState::BTN_UP:
                if (upAction) {
                    upAction();
                } else {
                    index = min(index + 1, numItems);
                    Menu::dispatchInput(state);
                }
                break;
            case JoystickState::BTN_DOWN:
                if (downAction) {
                    downAction();
                } else {
                    index = max(index - 1, 0);
                    Menu::dispatchInput(state);
                }
            case JoystickState::BTN_LEFT:
                if (leftAction) {
                    leftAction();
                } else {
                    Menu::dispatchInput(state);
                }
                break;
            case JoystickState::BTN_RIGHT:
                if (rightAction) {
                    rightAction();
                } else {
                    Menu::dispatchInput(state);
                }
                break;
            case JoystickState::BTN_ENTER:
                if (enterAction) {
                    enterAction();
                } else {
                    Menu::dispatchInput(state);
                }
            default:
                Menu::dispatchInput(state);
                break;
        }
#ifdef DEBUG
        Serial.println("PlayerListMenu - dispatch - Ende");
#endif
    };

    void addMenuItem(MenuItem *item) {
        numItems++;
        Menu::addMenuItem(item);
    }

private:
    std::function<void(void)> leftAction;
    std::function<void(void)> rightAction;
    std::function<void(void)> downAction;
    std::function<void(void)> upAction;
    std::function<void(void)> enterAction;
    unsigned short index = 0;
    short numItems = 0;
};


#endif //GPN_LASERTAG_PLAYERMENU_HPP
